package ufrn.imd.controller.impl;

import lombok.extern.java.Log;
import ufrn.imd.controller.Controller;
import ufrn.imd.domain.Client;
import ufrn.imd.service.impl.WithdrawalServiceImpl;

import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayDeque;
import java.util.Optional;
import java.util.Queue;

import static ufrn.imd.utils.NameSpace.HOST_WITHDRAWAL_SERVICE;
import static ufrn.imd.utils.NameSpace.PORT_WITHDRAWAL_SERVICE;

@Log
public class WithdrawalControllerImpl extends UnicastRemoteObject implements Controller {
    private volatile Queue<Client> clients = new ArrayDeque<>();

    public WithdrawalControllerImpl(WithdrawalServiceImpl service) throws RemoteException, MalformedURLException, AlreadyBoundException {
        super();
        log.info("Starting Withdrawal service!");
        new Notify(service).start();
        log.info(String.format("Initializing server in %s", HOST_WITHDRAWAL_SERVICE));
        LocateRegistry.createRegistry(PORT_WITHDRAWAL_SERVICE);
        Naming.bind(HOST_WITHDRAWAL_SERVICE, this);
    }

    @Override
    public void registerClient(Client client) throws RemoteException {
        clients.add(client);
        log.info("new client registred");
        log.info(String.format("total users: %d", clients.size()));
    }

    private class Notify extends Thread {
        private final WithdrawalServiceImpl service;

        public Notify(WithdrawalServiceImpl service) {
            super();
            this.service = service;
        }

        public void run() {
            for(;;) {
                if(clients.size() > 0) {
                    log.info("notyfing the clients!");
                    int i = 0;
                    clients.stream().forEach(client -> {
                        try {
                            log.info(String.format("Clients waiting %d", clients.size()));
                            this.service.withdraw(50.00, Optional.of(client));
                        } catch (RemoteException | RuntimeException e) {
                            e.printStackTrace();
                        } finally {
                            clients.remove();
                            log.info(String.format("Clients waiting %d", clients.size()));
                        }
                    });

                    try {
                        Thread.sleep(15 * 1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
}
